package com.willalp.web.controller.clockingin;

import com.willalp.common.annotation.Log;
import com.willalp.common.core.controller.BaseController;
import com.willalp.common.core.domain.AjaxResult;
import com.willalp.common.core.domain.entity.SysDictData;
import com.willalp.common.core.page.TableDataInfo;
import com.willalp.common.enums.BusinessType;
import com.willalp.common.utils.DateUtils;
import com.willalp.common.utils.SecurityUtils;
import com.willalp.common.utils.StringUtils;
import com.willalp.common.utils.poi.ExcelUtil;
import com.willalp.event.domain.CondoleRecord;
import com.willalp.event.service.ICondoleReordService;
import com.willalp.push.service.IHsAppPushService;
import com.willalp.system.service.ISysConfigService;
import com.willalp.system.service.ISysDictDataService;
import com.willalp.system.service.ISysDictTypeService;
import com.willalp.wx.domain.WxMessageBeanApprovalPass;
import com.willalp.wx.enums.WxTemplateKeyEnum;
import com.willalp.wx.service.WxMsgService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 慰问记录Controller
 *
 * @author willalp
 * @date 2023-05-11
 */
@RestController
@RequestMapping("/event/condole")
public class CondoleReordController extends BaseController {
    @Resource
    private ICondoleReordService condoleReordService;

    @Resource
    private WxMsgService wxMsgService;

    @Resource
    private IHsAppPushService pushService;

    /**
     * 查询慰问记录列表
     */
    @PreAuthorize("@ss.hasPermi('event:condole:list')")
    @GetMapping("/list")
    public TableDataInfo list(CondoleRecord condoleRecord) {
        startPage();
        List<CondoleRecord> list = condoleReordService.selectCondoleReordList(condoleRecord);
        return getDataTable(list);
    }

    @PostMapping("/reject")
    public AjaxResult reject(@RequestBody CondoleRecord condoleRecord) {
        condoleRecord.setBohui("已驳回");
        if (condoleReordService.updateById(condoleRecord)) {
            String title = "";
            switch (condoleRecord.getWwlx()) {
                case "1":
                    title = "常规住院";
                    break;
                case "2":
                    title = "大病住院";
                    break;
                case "3":
                    title = "人身伤害";
                    break;
                default:
                    title = "关心关爱";
                    break;
            }
            String openid = pushService.getOpenIdByUserNo(condoleRecord.getCreateBy());
            WxMessageBeanApprovalPass wxMessageBeanApprovalWait = new WxMessageBeanApprovalPass(
                    "您的" + title + "已被驳回", condoleRecord.getUserName(), DateUtils.getTime());
            wxMsgService.sendQRCodeMsg(openid, wxMessageBeanApprovalWait, WxTemplateKeyEnum.APPROVAL_RESULT);
        }
        return AjaxResult.success();
    }

    @Resource
    private ISysDictTypeService dictTypeService;

    /**
     * 导出慰问记录列表
     */
    @Log(title = "慰问记录", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(CondoleRecord condoleRecord) {
        List<CondoleRecord> list = condoleReordService.selectCondoleReordList(condoleRecord);
        List<SysDictData> condoleType = dictTypeService.selectDictDataByType("condole_type");
        for (CondoleRecord record : list) {
            if (null != record) {
                if(StringUtils.isNotBlank(record.getWwlx())) {
                    record.setWwlx(importTranslation(condoleType, record.getWwlx()));
                }
            }
        }
        ExcelUtil<CondoleRecord> util = new ExcelUtil<CondoleRecord>(CondoleRecord.class);
        return util.exportExcel(list, "慰问记录数据");
    }

    private String importTranslation(List<SysDictData> list, String value) {
        if (StringUtils.isNotBlank(value)) {
            if (0 != list.size()) {
                List<SysDictData> collect = list.stream()
                        .filter(item -> StringUtils.isNotBlank(item.getDictValue()) && item.getDictValue().equals(value))
                        .collect(Collectors.toList());
                if (0 != collect.size()) {
                    return collect.get(0).getDictLabel();
                }
            }
        }
        return "";
    }

    /**
     * 获取慰问记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('event:condole:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success(condoleReordService.getById(id));
    }

    /**
     * 新增慰问记录
     */
    @PreAuthorize("@ss.hasPermi('event:condole:add')")
    @Log(title = "慰问记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CondoleRecord condoleRecord) {
        condoleRecord.setCreateBy(SecurityUtils.getUsername());
        condoleRecord.setCreateTime(DateUtils.getNowDate());
        return toAjax(condoleReordService.save(condoleRecord));
    }

    /**
     * 修改慰问记录
     */
    @PreAuthorize("@ss.hasPermi('event:condole:edit')")
    @Log(title = "慰问记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CondoleRecord condoleRecord) {
        condoleRecord.setUpdateBy(SecurityUtils.getUsername());
        condoleRecord.setUpdateTime(DateUtils.getNowDate());
        return toAjax(condoleReordService.updateById(condoleRecord));
    }

    /**
     * 删除慰问记录
     */
    @PreAuthorize("@ss.hasPermi('event:condole:remove')")
    @Log(title = "慰问记录", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(condoleReordService.removeByIds(Arrays.asList(ids)));
    }
}
